# Generated by Django 5.2 on 2025-05-10 15:02

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nyxboard", "0002_healthcheck_next_check_time"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthcheck",
            name="processing_started_at",
            field=models.PositiveIntegerField(
                default=0,
                help_text="Unix timestamp when the check started processing",
                verbose_name="Processing Started At",
            ),
        ),
    ]



from dataclasses import dataclass


class Event:
    """Base class for all events."""


@dataclass
class CheckSucceeded(Event):
    """Check succeeded."""

    check_id: int


@dataclass
class CheckFailed(Event):
    """Check failed."""

    check_id: int
    result: bool


@dataclass
class ServiceStatusChanged(Event):
    """Service status changed."""

    service_id: int
    status: str



import logging
from typing import Callable

from ..domain import Event, Command
from .unit_of_work import UnitOfWork

logger = logging.getLogger(__name__)

Message = Event | Command


class MessageBus:
    """
    This is a simple implementation of a message bus that can handle
    messages of type Event and Command using event_handlers and command_handlers.
    It uses a UnitOfWork to collect new events.
    """

    def __init__(
        self,
        uow: UnitOfWork,
        event_handlers: dict[type[Event], list[Callable]],
        command_handlers: dict[type[Command], Callable],
    ):
        self.uow = uow
        self.event_handlers = event_handlers
        self.command_handlers = command_handlers
        self.queue: list[Message] = []

    def handle(self, message: Message):
        self.queue.append(message)
        while self.queue:
            message = self.queue.pop(0)
            if isinstance(message, Event):
                self.handle_event(message)
            elif isinstance(message, Command):
                self.handle_command(message)
            else:
                raise Exception(f"Cannot handle message of type {message}")

    def handle_event(self, event: Event):
        for handler in self.event_handlers[type(event)]:
            try:
                logger.debug("handling event %s with handler %s", event, handler)
                handler(event)
                self.queue.extend(self.uow.collect_new_events())
            except Exception:
                logger.exception("Exception handling event %s", event)
                continue

    def handle_command(self, command: Command):
        logger.debug("handling command %s", command)
        try:
            handler = self.command_handlers[type(command)]
            handler(command)
            self.queue.extend(self.uow.collect_new_events())
        except Exception:
            logger.exception("Exception handling command %s", command)
            raise



import time
import anyio
import logging
import threading

from typing import Protocol
from contextlib import asynccontextmanager

from anyio import to_thread
from anyio.from_thread import BlockingPortalProvider

from ..domain import Auto
from ..domain.commands import ExecuteChecks, StartCollector, StopCollector
from ..service_layer import MessageBus

logger = logging.getLogger(__name__)


class CheckCollector(Protocol):
    """A protocol for a check collector."""

    def __init__(self, *, interval: int = 1) -> None: ...

    def start(self) -> None:
        """Start the collector."""
        ...

    def stop(self) -> None:
        """Stop the collector."""
        ...

    def set_portal_provider(self, portal_provider) -> None:
        """Set the portal provider for the collector."""
        pass

    def set_message_bus(self, bus: MessageBus) -> None:
        """Set the message bus for the collector."""


@asynccontextmanager
async def running_collector(bus):
    """Context manager for collector lifecycle"""
    bus.handle(StartCollector())
    try:
        yield
    finally:
        bus.handle(StopCollector())
        # Optional: wait a bit for collector to shut down cleanly
        await anyio.sleep(0.1)


class AsyncCheckCollector(CheckCollector):
    def __init__(self, *, interval: int = 1) -> None:
        self.interval = interval
        self._running = False
        self._thread = Auto
        self._bus = Auto

    def set_portal_provider(self, portal_provider: BlockingPortalProvider) -> None:
        """Set the portal provider for the collector."""
        self._portal_provider = portal_provider

    def set_message_bus(self, bus: MessageBus) -> None:
        """Set the message bus for the collector."""
        self._bus = bus

    async def _async_start(self):
        if self._running:
            return
        if self._bus is None:
            raise ValueError(
                "Message bus is not set. Please set the message bus before starting the collector."
            )
        self._running = True
        i = 0
        while self._running:
            checks = await self._bus.uow.store.checks.list_due_checks_async()
            print("due checks: ", checks)
            if len(checks) > 0:
                # Use a worker thread to run the checks
                await to_thread.run_sync(self._bus.handle, ExecuteChecks(checks=checks))
            i += 1
            await anyio.sleep(self.interval)

    def start(self) -> None:
        thread = threading.Thread(
            target=self._start_in_thread,
            daemon=True,  # Make it a daemon thread so it doesn't block program exit
        )
        thread.start()
        self._thread = thread
        logger.debug("check collector started!")

    def _start_in_thread(self) -> None:
        """Run the collector in a thread."""
        with self._portal_provider as portal:
            portal.start_task_soon(self._async_start)
            # This thread will keep running as long as the portal is alive
            # Add some way to join/exit this thread when needed
            while self._running:
                time.sleep(1)  # Keep thread alive but don't consume CPU

    def stop(self):
        if not self._running:
            return

        self._running = False

        # Wait for the thread to finish if it exists
        if self._thread is not None and self._thread.is_alive():
            self._thread.join(timeout=2.0)  # Wait up to 2 seconds

        # Log or handle if thread didn't exit cleanly
        if self._thread and self._thread.is_alive():
            logger.warning("Warning: Collector thread didn't exit cleanly")
        logger.debug("check collector stopped!")



from dataclasses import dataclass, field

from .models import Check, CheckResult


class Command:
    """Base class for all commands."""


@dataclass
class ExecuteChecks(Command):
    """Run all pending checks."""

    checks: list[Check] = field(default_factory=list)


@dataclass
class RegisterCheck(Command):
    """Register a check."""

    check_id: int
    check_type: str
    check_data: dict


@dataclass
class DeleteCheck(Command):
    """Delete a check."""

    check_id: int


@dataclass
class AddCheck(Command):
    """Add a check."""

    check: Check


@dataclass
class AddCheckResult(Command):
    """Add a result for a check."""

    check_result: CheckResult


@dataclass
class StartCollector(Command):
    """Start the collector."""

    pass


@dataclass
class StopCollector(Command):
    """Stop the collector."""

    pass



import inspect

from anyio.from_thread import BlockingPortalProvider

from .adapters.runner import CheckRunner, AsyncCheckRunner
from .domain import Auto
from .adapters.collector import CheckCollector, AsyncCheckCollector
from .adapters.repositories import RepositoryStore, InMemoryStore
from .adapters.notification import Notifier, LoggingNotifier
from .service_layer import handlers, UnitOfWork, MessageBus


def inject_dependencies(handler, dependencies):
    params = inspect.signature(handler).parameters
    deps = {
        name: dependency for name, dependency in dependencies.items() if name in params
    }
    return lambda message: handler(message, **deps)


def bootstrap(
    uow: UnitOfWork = Auto,
    portal_provider: BlockingPortalProvider = Auto,
    store: RepositoryStore = Auto,
    collector: CheckCollector = Auto,
    runner: CheckRunner = Auto,
    notifier: Notifier = Auto,
) -> MessageBus:
    """Creates a new MessageBus instance with all dependencies injected."""
    if not store:
        store = InMemoryStore()

    if not uow:
        uow = UnitOfWork(store=store)

    if not portal_provider:
        portal_provider = BlockingPortalProvider()

    if hasattr(store, "set_portal_provider"):
        store.set_portal_provider(portal_provider)

    if not collector:
        collector = AsyncCheckCollector(interval=1)

    if not runner:
        runner = AsyncCheckRunner(portal_provider=portal_provider)

    if not notifier:
        # Use logging notifier by default
        notifier = LoggingNotifier()

    if hasattr(notifier, "set_portal_provider"):
        notifier.set_portal_provider(portal_provider)

    dependencies = {
        "uow": uow,
        "portal_provider": portal_provider,
        "collector": collector,
        "runner": runner,
        "notifier": notifier,
    }
    injected_event_handlers = {
        event_type: [
            inject_dependencies(handler, dependencies) for handler in event_handlers
        ]
        for event_type, event_handlers in handlers.EVENT_HANDLERS.items()
    }
    injected_command_handlers = {
        command_type: inject_dependencies(handler, dependencies)
        for command_type, handler in handlers.COMMAND_HANDLERS.items()
    }
    bus = MessageBus(
        uow=uow,
        event_handlers=injected_event_handlers,
        command_handlers=injected_command_handlers,
    )
    collector.set_message_bus(bus)
    return bus



from ...domain import Result, Check, Service
from .interface import (
    Repository,
    ResultRepository,
    CheckRepository,
    ServiceRepository,
    RepositoryStore,
)


class InMemoryResultRepository(ResultRepository):
    """An in-memory implementation of the ResultRepository interface."""

    def __init__(self) -> None:
        self.results: dict[int, Result] = {}
        self.seen: set[Result] = set()

    def add(self, result: Result) -> None:
        if result.result_id is None:
            result.result_id = len(self.results)
        self.results[result.result_id] = result
        self.seen.add(result)

    def get(self, result_id: int) -> Result:
        return self.results[result_id]

    def list(self) -> list[Result]:
        return list(self.results.values())


class InMemoryCheckRepository(CheckRepository):
    """An in-memory implementation of the CheckRepository interface."""

    def __init__(self) -> None:
        self.checks: dict[int, Check] = {}
        self.seen: set[Check] = set()

    def add(self, check: Check) -> None:
        self.checks[check.check_id] = check
        self.seen.add(check)

    def get(self, check_id: int) -> Check:
        return self.checks[check_id]

    def list(self) -> list[Check]:
        return list(self.checks.values())


class InMemoryServiceRepository(ServiceRepository):
    """An in-memory implementation of the ServiceRepository interface."""

    def __init__(self) -> None:
        self.services: dict[int, Service] = {}
        self.seen: set[Service] = set()

    def add(self, service: Service) -> None:
        self.services[service.service_id] = service
        self.seen.add(service)

    def get(self, service_id: int) -> Service:
        return self.services[service_id]

    def list(self) -> list[Service]:
        return list(self.services.values())


class InMemoryStore(RepositoryStore):
    """An in-memory store for the repositories."""

    def __init__(
        self,
        *,
        results: InMemoryResultRepository = InMemoryResultRepository(),
        checks: InMemoryCheckRepository = InMemoryCheckRepository(),
        services: InMemoryServiceRepository = InMemoryServiceRepository(),
    ) -> None:
        self.results = results
        self.checks = checks
        self.services = services

    def list(self) -> list[Repository]:
        return [
            self.results,
            self.checks,
            self.services,
        ]



import os
import httpx
import logging

from typing import Protocol
from anyio.from_thread import BlockingPortalProvider

from ..domain.models import Check, Result, Service

logger = logging.getLogger(__name__)


class Notifier(Protocol):
    """Interface for notification services."""

    def notify_check_failed(self, check: Check, result: Result) -> None:
        """Notify about a failed check."""
        ...

    def notify_service_status_changed(self, service: Service, status: str) -> None:
        """Notify about a service status change."""
        ...


class AsyncTelegramNotifier(Notifier):
    def __init__(self, token: str | None = None, chat_id: str | None = None):
        self.token = token or os.environ.get("TELEGRAM_BOT_TOKEN")
        self.chat_id = chat_id or os.environ.get("TELEGRAM_CHAT_ID")
        if not self.token or not self.chat_id:
            logger.warning("Telegram notifier initialized without token or chat_id")
        self.url = (
            f"https://api.telegram.org/bot{self.token}/sendMessage"
            if self.token
            else ""
        )
        self.portal_provider: BlockingPortalProvider | None = None

    def set_portal_provider(self, portal_provider: BlockingPortalProvider) -> None:
        """Set the portal provider for async operations."""
        self.portal_provider = portal_provider

    async def async_send(self, text: str, high_priority: bool = False) -> None:
        """Send a message via Telegram asynchronously."""
        if not self.token or not self.chat_id or not self.url:
            logger.warning("Cannot send Telegram notification: missing credentials")
            return

        try:
            payload = {
                "chat_id": self.chat_id,
                "text": text,
                "parse_mode": "MarkdownV2",
                "disable_notification": not high_priority,
            }
            async with httpx.AsyncClient() as client:
                resp = await client.post(self.url, data=payload, timeout=10.0)
                resp.raise_for_status()
        except Exception as e:
            logger.error(f"Failed to send Telegram notification: {e}")

    @staticmethod
    def escape_markdown_v2(text: str) -> str:
        """Escape special characters for Telegram's MarkdownV2 format."""
        # Characters that need escaping in MarkdownV2: _ * [ ] ( ) ~ ` > # + - = | { } . !
        special_chars = [
            "_",
            "*",
            "[",
            "]",
            "(",
            ")",
            "~",
            "`",
            ">",
            "#",
            "+",
            "-",
            "=",
            "|",
            "{",
            "}",
            ".",
            "!",
        ]
        escaped_text = text
        for char in special_chars:
            escaped_text = escaped_text.replace(char, f"\\{char}")
        return escaped_text

    async def async_notify_check_failed(self, check: Check, result: Result) -> None:
        """Notify about a failed check asynchronously."""
        error_msg = result.data.get("error_msg", "Unknown error")
        error_type = result.data.get("error_type", "")
        status_code = result.data.get("status_code", "")

        # Escape all text for MarkdownV2
        escaped_name = (
            self.escape_markdown_v2(check.name) if check.name else "Unnamed Check"
        )
        escaped_url = self.escape_markdown_v2(check.url)
        escaped_error_msg = self.escape_markdown_v2(str(error_msg))
        escaped_error_type = self.escape_markdown_v2(str(error_type))

        message = "🔴 *Check Failed*\n"
        message += f"Name: {escaped_name}\n"
        message += f"URL: {escaped_url}\n"
        if status_code:
            message += f"Status: {status_code}\n"
        if error_type:
            message += f"Error Type: {escaped_error_type}\n"
        message += f"Error: {escaped_error_msg}"

        await self.async_send(message, high_priority=True)

    async def async_notify_service_status_changed(
        self, service: Service, status: str
    ) -> None:
        """Notify about a service status change asynchronously."""
        service_name = service.data.get("name", f"Service {service.service_id}")
        escaped_service_name = self.escape_markdown_v2(service_name)
        escaped_status = self.escape_markdown_v2(status)

        emoji = (
            "🔴"
            if status.lower() == "down"
            else "🟢"
            if status.lower() == "up"
            else "⚠️"
        )
        message = f"{emoji} *Service Status Changed*\n"
        message += f"Service: {escaped_service_name}\n"
        message += f"Status: {escaped_status}"

        await self.async_send(message, high_priority=True)

    # Sync methods that call async methods through the portal
    def notify_check_failed(self, check: Check, result: Result) -> None:
        """Notify about a failed check."""
        if self.portal_provider is None:
            logger.warning("Cannot send notification: portal provider not set")
            return

        with self.portal_provider as portal:
            portal.call(self.async_notify_check_failed, check, result)

    def notify_service_status_changed(self, service: Service, status: str) -> None:
        """Notify about a service status change."""
        if self.portal_provider is None:
            logger.warning("Cannot send notification: portal provider not set")
            return

        with self.portal_provider as portal:
            portal.call(self.async_notify_service_status_changed, service, status)


class LoggingNotifier(Notifier):
    """A simple notifier that logs messages to the console."""

    def notify_check_failed(self, check: Check, result: Result) -> None:
        """Log a failed check notification."""
        check_name = check.name if check.name else f"Check {check.check_id}"
        logger.error(
            f"Check failed: {check_name} (ID: {check.check_id}), Result: {result}"
        )

    def notify_service_status_changed(self, service: Service, status: str) -> None:
        """Log a service status change notification."""
        logger.info(f"Service status changed: {service.service_id}, Status: {status}")

    def set_portal_provider(self, portal_provider: BlockingPortalProvider) -> None:
        """Set the portal provider for async operations."""
        self._portal_provider = portal_provider



from .events import Event
from .commands import Command
from .constants import Auto
from .models import (
    Result,
    Check,
    CheckResult,
    CheckStatus,
    CheckType,
    Service,
    ResultStatus,
    ResultStatusType,
    StatusChoices,
)


__all__ = [
    "Auto",
    "Check",
    "CheckResult",
    "CheckStatus",
    "CheckType",
    "Command",
    "Event",
    "Result",
    "ResultStatus",
    "ResultStatusType",
    "StatusChoices",
    "Service",
]



"""
CLI entrypoints for developer tools.
"""

import os
import fnmatch
import argparse
from pathlib import Path


def llm_content():
    """
    Output all relevant code / documentation in the project including
    the relative path and content of each file.
    """

    def echo_filename_and_content(files):
        """Print the relative path and content of each file."""
        for f in files:
            print(f)
            contents = f.read_text()
            relative_path = f.relative_to(project_root)
            print(relative_path)
            print("---")
            print(contents)
            print("---")

    parser = argparse.ArgumentParser(
        description="Output project code/documentation for LLM context"
    )
    parser.add_argument(
        "--exclude-dirs",
        nargs="+",
        default=[".venv"],
        help="Directories to exclude",
    )
    parser.add_argument(
        "--patterns",
        nargs="+",
        default=["*.py", "*.rst", "*.js", "*.ts", "*.html"],
        help="File patterns to include",
    )

    args = parser.parse_args()

    project_root = Path.cwd()
    # Exclude files and directories. This is tuned to make the project fit into the
    # 200k token limit of the claude 3 models.
    exclude_files = {}
    exclude_dirs = set(args.exclude_dirs)
    patterns = args.patterns
    all_files = []

    for root, dirs, files in os.walk(project_root):
        root = Path(root)
        # d is the plain directory name
        dirs[:] = [d for d in dirs if d not in exclude_dirs]
        for pattern in patterns:
            for filename in fnmatch.filter(files, pattern):
                if filename not in exclude_files:
                    all_files.append(root / filename)

    echo_filename_and_content(all_files)



import time
from typing import Literal, TypeAlias

from .events import Event


class ResultStatus:
    OK: Literal["ok"] = "ok"
    ERROR: Literal["error"] = "error"


ResultStatusType: TypeAlias = Literal["ok", "error"]


class StatusChoices:
    PASSED: Literal["passed"] = "passed"
    FAILED: Literal["failed"] = "failed"
    WARNING: Literal["warning"] = "warning"
    RECOVERING: Literal["recovering"] = "recovering"
    UNKNOWN: Literal["unknown"] = "unknown"

    @classmethod
    def get_css_class(cls, status: str) -> str:
        css_classes = {
            cls.PASSED: "status-passed",
            cls.FAILED: "status-failed",
            cls.WARNING: "status-warning",
            cls.RECOVERING: "status-recovering",
            cls.UNKNOWN: "status-unknown",
        }
        return css_classes.get(status, "")


StatusType: TypeAlias = Literal["passed", "failed", "warning", "recovering", "unknown"]


class Result:
    def __init__(
        self,
        *,
        result_id: int | None = None,
        check_id: int,
        status: ResultStatusType,
        data: dict,
    ) -> None:
        self.result_id = result_id
        self.check_id = check_id
        self.status = status
        self.data = data
        self.events: list["Event"] = []

    def __repr__(self) -> str:
        return f"Result(result_id={self.result_id}, check_id={self.check_id} status={self.status}, data={self.data})"


class CheckStatus:
    IDLE: Literal["idle"] = "idle"
    PROCESSING: Literal["processing"] = "processing"


CheckStatusType: TypeAlias = Literal["idle", "processing"]


class CheckType:
    HTTP: Literal["http"] = "http"
    JSON_HTTP: Literal["json-http"] = "json-http"
    TCP: Literal["tcp"] = "tcp"
    PING: Literal["ping"] = "ping"
    DNS: Literal["dns"] = "dns"
    CUSTOM: Literal["custom"] = "custom"


CheckTypeType: TypeAlias = Literal["http", "json-http", "tcp", "ping", "dns", "custom"]


class Check:
    def __init__(
        self,
        *,
        check_id: int,
        service_id: int,
        name: str = "",
        check_type: str,
        url: str,
        check_interval: int = 300,
        next_check_time: int = 0,
        processing_started_at: int = 0,
        status: CheckStatusType = CheckStatus.IDLE,
        disabled: bool = False,
        data: dict,
    ) -> None:
        self.check_id = check_id
        self.service_id = service_id
        self.name = name
        self.check_type = check_type
        self.url = url
        self.check_interval = check_interval
        self.next_check_time = next_check_time
        self.processing_started_at = processing_started_at
        self.status = status
        self.disabled = disabled
        self.data = data
        self.events: list["Event"] = []

        # Will be populated when check is executed
        self.result: "Result" = None  # type: ignore

    def __repr__(self) -> str:
        return f"Check(check={self.check_id}, name='{self.name}', service_id={self.service_id} url={self.url})"

    def execute(self) -> None:
        # Logic to execute the check
        pass

    def schedule_next_check(self) -> None:
        """Schedule the next execution of this check."""
        current_time = int(time.time())

        self.next_check_time = current_time + self.check_interval
        self.status = CheckStatus.IDLE
        self.processing_started_at = 0


class CheckResult:
    def __init__(self, check: Check, result: Result) -> None:
        self.check = check
        self.result = result
        self.events: list["Event"] = []

    @property
    def passed(self) -> bool:
        return self.result.status == ResultStatus.OK

    @property
    def should_notify(self) -> bool:
        return self.result.status == ResultStatus.ERROR


class Service:
    def __init__(self, *, service_id: int, data: dict) -> None:
        self.service_id = service_id
        self.data = data
        self.events: list["Event"] = []



# Generated by Django 5.2 on 2025-05-10 14:56

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nyxboard", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthcheck",
            name="next_check_time",
            field=models.PositiveIntegerField(
                default=0,
                help_text="Unix timestamp of the next check",
                verbose_name="Next Check Time",
            ),
        ),
    ]



import sqlite3
import json
import logging
import time
from typing import List
import anyio
import aiosqlite

from pathlib import Path

from anyio.from_thread import BlockingPortalProvider

from ...domain import Check, Result, Service
from .interface import (
    RepositoryStore,
    CheckRepository,
    ResultRepository,
    ServiceRepository,
)

logger = logging.getLogger(__name__)


def row_to_check(row: aiosqlite.Row) -> Check:
    check_id = row["id"]
    service_id = row["service_id"]
    name = row["name"]
    check_type = row["check_type"]
    url = row["url"]
    check_interval = row["check_interval"]
    next_check_time = row["next_check_time"]
    processing_started_at = row["processing_started_at"]
    status = row["status"]
    disabled = bool(row["disabled"])  # SQLite stores booleans as 0/1
    check = Check(
        check_id=check_id,
        service_id=service_id,
        name=name,
        check_type=check_type,
        url=url,
        check_interval=check_interval,
        next_check_time=next_check_time,
        processing_started_at=processing_started_at,
        status=status,
        disabled=disabled,
        data={},
    )
    return check


class SqliteCheckRepository(CheckRepository):
    def __init__(self, db_path: Path) -> None:
        self._db_path = db_path
        self._portal_provider: BlockingPortalProvider | None = None
        self._schema_ready = False
        self.seen: set[Check] = set()

    # ---------- öffentliche, SYNCHRONE Ports ----------
    def get(self, check_id: int) -> Check:
        """Get a check from the repository by ID."""
        return self._await(self._get_async(check_id))

    def list(self) -> List[Check]:
        return self._await(self.list_async())

    def add(self, check: Check) -> None:
        if self._portal_provider is None:
            return  # No portal provider set, cannot add check
        with self._portal_provider as portal:
            portal.call(self._add_async, check)

    # ---------- interne async-Implementierung ----------
    async def _get_async(self, check_id: int) -> Check:
        """Get a check from the repository by ID asynchronously."""
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)
            db.row_factory = aiosqlite.Row
            [row] = await db.execute_fetchall(
                "SELECT id, service_id, name, check_type, url, check_interval, next_check_time, processing_started_at, status, disabled FROM health_check WHERE id = ?",
                (check_id,),
            )
            if row is None:
                raise KeyError(f"Check with ID {check_id} not found")
            return row_to_check(row)

    async def list_async(self) -> List[Check]:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            db.row_factory = aiosqlite.Row
            rows = await db.execute_fetchall(
                "SELECT id, service_id, name, check_type, url, check_interval, next_check_time, processing_started_at, status, disabled FROM health_check"
            )
            return [row_to_check(r) for r in rows]

    async def list_due_checks_async(self) -> List[Check]:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            current_time = int(time.time())
            db.row_factory = aiosqlite.Row

            # Single atomic operation to find and claim checks
            # Using SQLite's RETURNING clause (available in SQLite 3.35.0+)
            result = await db.execute(
                """UPDATE health_check
                   SET status                = 'processing',
                       processing_started_at = ?
                   WHERE id IN (SELECT id
                                FROM health_check
                                WHERE next_check_time <= ?
                                  AND status = 'idle'
                                  AND disabled = 0
                                LIMIT 100 -- Optional: process a batch at a time
                   )
                   RETURNING id, service_id, name, check_type, url, check_interval, next_check_time, processing_started_at, status, disabled""",
                (current_time, current_time),
            )

            rows = await result.fetchall()
            await db.commit()

            return [row_to_check(r) for r in rows]

    async def _add_async(self, check: Check) -> None:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            await db.execute(
                """INSERT OR REPLACE INTO health_check
                   (id, service_id, name, check_type, url, check_interval, 
                    status, next_check_time, processing_started_at, disabled)
                   VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)""",
                (
                    check.check_id,
                    check.service_id,
                    check.name,
                    check.check_type,
                    check.url,
                    check.check_interval,
                    check.status,
                    check.next_check_time,
                    check.processing_started_at,
                    int(check.disabled),  # Convert bool to int for SQLite
                ),
            )
            await db.commit()

    # ---------- Bridge sync → async ----------
    def _await(self, coro):
        async def _run():
            return await coro  # Coroutine tatsächlich ausführen

        return anyio.from_thread.run(_run)  # Callable (!) an from_thread.run()

    # ---------- einmalige Schema-Initialisierung ----------

    async def _ensure_schema(self, db: aiosqlite.Connection) -> None:
        if self._schema_ready:
            return
        await db.executescript(
            """
            CREATE TABLE IF NOT EXISTS health_check (
                id               INTEGER PRIMARY KEY,
                service_id       INTEGER NOT NULL,
                name             TEXT    DEFAULT '',
                check_type       TEXT    NOT NULL,
                url              TEXT    NOT NULL,
                check_interval   INTEGER NOT NULL,
                status           TEXT    DEFAULT 'idle',
                next_check_time  INTEGER DEFAULT 0,
                processing_started_at INTEGER DEFAULT 0,
                disabled         INTEGER DEFAULT 0
            );
            """
        )
        await db.commit()
        self._schema_ready = True


class SqliteResultRepository(ResultRepository):
    def __init__(self, db_path: Path) -> None:
        self._db_path = db_path
        self._schema_ready = False
        self.seen = set()
        self._portal_provider: BlockingPortalProvider | None = None

    # ---------- öffentliche, SYNCHRONE Ports ----------
    def add(self, result: Result) -> None:
        if self._portal_provider is None:
            return  # No portal provider set, cannot add a result
        with self._portal_provider as portal:
            portal.call(self._add_async, result)

    def get(self, result_id: int) -> Result:
        return self._await(self._get_async(result_id))

    def list(self) -> List[Result]:
        return self._await(self._list_async())

    # ---------- interne async-Implementierung ----------
    async def _add_async(self, result: Result) -> None:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)
            await db.execute(
                """INSERT INTO check_result (id, health_check_id, status, data, created_at)
                   VALUES (?, ?, ?, ?, datetime('now'))""",
                (
                    result.result_id,
                    result.check_id,
                    result.status,
                    json.dumps(result.data),
                ),
            )
            await db.commit()
            self.seen.add(result)

    async def _get_async(self, result_id: int) -> Result:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)
            db.row_factory = aiosqlite.Row
            [row] = await db.execute_fetchall(
                "SELECT id, health_check_id, status, data FROM check_result WHERE id = ?",
                (result_id,),
            )
            if row is None:
                raise KeyError(f"Result with ID {result_id} not found")
            return Result(
                result_id=row["id"],
                check_id=row["check_id"],
                status=row["status"],
                data=json.loads(row["data"]),
            )

    async def _list_async(self) -> List[Result]:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)
            db.row_factory = aiosqlite.Row
            rows = await db.execute_fetchall(
                "SELECT id, health_check_id, status, data FROM check_result"
            )
            return [
                Result(
                    result_id=row["id"],
                    check_id=row["health_check_id"],
                    status=row["status"],
                    data=json.loads(row["data"]),
                )
                for row in rows
            ]

    # ---------- Bridge sync → async ----------
    def _await(self, coro):
        async def _run():
            return await coro

        return anyio.from_thread.run(_run)

    # ---------- einmalige Schema-Initialisierung ----------
    async def _ensure_schema(self, db: aiosqlite.Connection) -> None:
        if self._schema_ready:
            return
        await db.executescript(
            """
            CREATE TABLE IF NOT EXISTS check_result (
                id              INTEGER PRIMARY KEY,
                health_check_id INTEGER NOT NULL,
                status          TEXT NOT NULL,
                data            TEXT,
                created_at      TIMESTAMP DEFAULT CURRENT_TIMESTAMP
            );
            """
        )
        await db.commit()
        self._schema_ready = True


class SqliteServiceRepository(ServiceRepository):
    def __init__(self, db_path: Path) -> None:
        self._db_path = db_path
        self._portal_provider: BlockingPortalProvider | None = None
        self._schema_ready = False
        self.seen: set[Service] = set()

    # ---------- öffentliche, SYNCHRONE Ports ----------
    def list(self) -> List[Service]:
        return self._await(self.list_async())

    def add(self, service: Service) -> None:
        if self._portal_provider is None:
            return  # No portal provider set, cannot add service
        with self._portal_provider as portal:
            portal.call(self._add_async, service)

    def get(self, service_id: int) -> Service:
        return self._await(self._get_async(service_id))

    # ---------- interne async-Implementierung ----------
    async def list_async(self) -> List[Service]:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            db.row_factory = aiosqlite.Row
            rows = await db.execute_fetchall("SELECT id, name FROM service")
            services = []
            for row in rows:
                service_id, name = row
                data = {"name": name}
                services.append(Service(service_id=service_id, data=data))
            return services

    async def _add_async(self, service: Service) -> None:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            await db.execute(
                """INSERT OR REPLACE INTO service (id, name) VALUES (?, ?)""",
                (
                    service.service_id,
                    service.data.get("name", ""),
                ),
            )
            await db.commit()
            self.seen.add(service)

    async def _get_async(self, service_id: int) -> Service:
        async with aiosqlite.connect(self._db_path) as db:
            await self._ensure_schema(db)

            db.row_factory = aiosqlite.Row
            [row] = await db.execute_fetchall(
                "SELECT id, name FROM service WHERE id = ?", (service_id,)
            )
            if row is None:
                raise KeyError(f"Service with ID {service_id} not found")

            service_id, name = row
            data = {"name": name}
            return Service(service_id=service_id, data=data)

    # ---------- Bridge sync → async ----------
    def _await(self, coro):
        async def _run():
            return await coro  # Coroutine tatsächlich ausführen

        return anyio.from_thread.run(_run)  # Callable (!) an from_thread.run()

    # ---------- einmalige Schema-Initialisierung ----------
    async def _ensure_schema(self, db: aiosqlite.Connection) -> None:
        if self._schema_ready:
            return
        await db.executescript(
            """
            CREATE TABLE IF NOT EXISTS service
            (
                id   INTEGER PRIMARY KEY,
                name TEXT NOT NULL
            );
            """
        )
        await db.commit()
        self._schema_ready = True


class SqliteStore(RepositoryStore):
    def __init__(self, db_path: Path) -> None:
        self.db_path = db_path
        self._connection: sqlite3.Connection | None = None
        self._thread_id: int | None = None

        # Initialize repositories
        self.results = SqliteResultRepository(db_path)
        self.checks = SqliteCheckRepository(db_path)
        self.services = SqliteServiceRepository(db_path)

        # Blocking portal provider
        self._portal_provider: BlockingPortalProvider | None = None

    def set_portal_provider(self, portal_provider: BlockingPortalProvider) -> None:
        """Set the portal provider for the store."""
        self._portal_provider = portal_provider
        self.results._portal_provider = portal_provider
        self.checks._portal_provider = portal_provider
        self.services._portal_provider = portal_provider

    @property
    def connection(self) -> sqlite3.Connection:
        # Create a new connection for the current thread if needed
        import threading

        thread_id = threading.get_ident()
        if self._thread_id != thread_id or self._connection is None:
            # Close existing connection if from a different thread
            if self._connection is not None:
                try:
                    self._connection.close()
                except Exception:
                    pass

            # Create new connection for current thread
            conn = sqlite3.connect(self.db_path)
            conn.row_factory = sqlite3.Row
            self._connection = conn
            self._thread_id = thread_id
            logger.debug(f"Created new SQLite connection for thread {thread_id}")

        # At this point self._connection should never be None
        assert self._connection is not None
        return self._connection

    def list(self) -> List:
        return [
            self.results,
            self.checks,
            self.services,
        ]



from typing import Callable

from anyio.from_thread import BlockingPortalProvider

from ..adapters.collector import CheckCollector
from ..adapters.notification import Notifier
from ..domain import events, commands
from ..domain.models import CheckResult
from ..adapters.runner import CheckRunner
from .unit_of_work import UnitOfWork
from ..domain.commands import AddCheckResult


def execute_checks(
    cmd: commands.ExecuteChecks, runner: CheckRunner, uow: UnitOfWork
) -> None:
    """Execute all pending checks."""
    check_by_check_id = {check.check_id: check for check in cmd.checks}

    def result_received(result):
        check = check_by_check_id[result.check_id]
        check.schedule_next_check()  # Schedule the next check after a result is received
        check_result = CheckResult(check=check, result=result)
        inner_cmd = AddCheckResult(check_result=check_result)
        uow.add_command(inner_cmd)  # add command to the unit of work

    runner.run_all(cmd.checks, result_received)


def add_check(cmd: commands.AddCheck, uow: UnitOfWork) -> None:
    """Add a check to the repository."""
    check = cmd.check
    with uow:
        uow.store.checks.add(check)
        uow.commit()


def add_check_result(
    cmd: commands.AddCheckResult, uow: UnitOfWork, notifier: Notifier
) -> None:
    """Add a check to the repository and trigger notifications if needed."""
    check_result = cmd.check_result
    check, result = check_result.check, check_result.result
    with uow:
        uow.store.results.add(result)
        uow.store.checks.add(check)
        uow.commit()

    # If the check_result should be notified, use the notifier to send notifications
    if check_result.should_notify:
        notifier.notify_check_failed(check, result)


def start_collector(
    _cmd: commands.StartCollector,
    collector: CheckCollector,
    portal_provider: BlockingPortalProvider,
) -> None:
    """Start the check collector."""
    collector.set_portal_provider(portal_provider)
    collector.start()


def stop_collector(_cmd: commands.StopCollector, collector: CheckCollector) -> None:
    """Stop the check collector."""
    collector.stop()


def service_status_changed(
    event: events.ServiceStatusChanged, uow: UnitOfWork, notifier: Notifier
) -> None:
    with uow:
        service = uow.store.services.get(event.service_id)
        # Notify about the service status change
        notifier.notify_service_status_changed(service, event.status)
        # Update service status
        if hasattr(service, "update_status"):
            service.update_status(event.status)
        uow.commit()


def check_failed(event: events.CheckFailed, uow: UnitOfWork) -> None:
    # This handler is called when a check fails, after the notification has already been sent
    # We could add additional actions here, like retrying the check or updating service status
    pass


def check_succeeded(event: events.CheckSucceeded, uow: UnitOfWork) -> None:
    # This handler is called when a check succeeds
    # We could add actions like resetting failure counters, etc.
    pass


EVENT_HANDLERS: dict[type[events.Event], list[Callable]] = {
    events.ServiceStatusChanged: [service_status_changed],
    events.CheckFailed: [check_failed],
    events.CheckSucceeded: [check_succeeded],
}

COMMAND_HANDLERS: dict[type[commands.Command], Callable] = {
    commands.ExecuteChecks: execute_checks,
    commands.AddCheck: add_check,
    commands.AddCheckResult: add_check_result,
    commands.StartCollector: start_collector,
    commands.StopCollector: stop_collector,
}



from collections.abc import Callable
from typing import Protocol, Iterable

from ...domain import Check


class CheckRunner(Protocol):
    """A runner interface for executing health checks."""

    def run_all(self, checks: Iterable[Check], result_received: Callable) -> None:
        """Run all checks."""
        ...



"""
Development data utilities for creating test services and checks.
"""

from typing import Dict, List

from django.utils import timezone

from nyxmon.domain import CheckStatus
from nyxboard.models import Service, HealthCheck


def create_dev_service(name: str) -> Service:
    """Create a development service with the given name."""
    service = Service.objects.create(name=name)
    return service


def create_dev_check(
    service: Service,
    name: str,
    check_type: str = "http",
    url: str = "http://localhost:8000/",
    interval: int = 60,
    disabled: bool = False,
) -> HealthCheck:
    """Create a development health check for the given service."""
    next_check_time = int(timezone.now().timestamp()) + interval

    check = HealthCheck.objects.create(
        service=service,
        name=name,
        check_type=check_type,
        url=url,
        check_interval=interval,
        status=CheckStatus.IDLE,
        next_check_time=next_check_time,
        disabled=disabled,
    )

    return check


def create_sample_data() -> Dict[str, List]:
    """Create sample development data including services and checks."""
    # Track created objects
    created: dict[str, list] = {
        "services": [],
        "checks": [],
    }

    # Create Dev Service
    dev_service = create_dev_service("Development Server")
    created["services"].append(dev_service)

    # Create working check - checks the dashboard
    working_check = create_dev_check(
        service=dev_service,
        name="Dashboard Check",
        url="http://localhost:8000/",
        interval=60,
    )
    created["checks"].append(working_check)

    # Create failing check - non-existent URL
    failing_check = create_dev_check(
        service=dev_service,
        name="Failing Check",
        url="http://localhost:8000/non-existent-url/",
        interval=60,
    )
    created["checks"].append(failing_check)

    return created



from django.urls import path
from . import views

app_name = "nyxboard"

urlpatterns = [
    # Dashboard
    path("", views.dashboard, name="dashboard"),
    # Service CRUD
    path("services/", views.service_list, name="service_list"),
    path("services/create/", views.service_create, name="service_create"),
    path("services/<int:service_id>/", views.service_detail, name="service_detail"),
    path(
        "services/<int:service_id>/update/", views.service_update, name="service_update"
    ),
    path(
        "services/<int:service_id>/delete/", views.service_delete, name="service_delete"
    ),
    # HealthCheck CRUD
    path("healthchecks/", views.healthcheck_list, name="healthcheck_list"),
    path("healthchecks/create/", views.healthcheck_create, name="healthcheck_create"),
    path(
        "services/<int:service_id>/healthchecks/create/",
        views.healthcheck_create,
        name="healthcheck_create_for_service",
    ),
    path(
        "healthchecks/<int:check_id>/",
        views.healthcheck_detail,
        name="healthcheck_detail",
    ),
    path(
        "healthchecks/<int:check_id>/update/",
        views.healthcheck_update,
        name="healthcheck_update",
    ),
    path(
        "healthchecks/<int:check_id>/delete/",
        views.healthcheck_delete,
        name="healthcheck_delete",
    ),
    # HTMX-enabled endpoints
    path(
        "healthchecks/<int:check_id>/status/",
        views.healthcheck_update_status,
        name="healthcheck_update_status",
    ),
    path(
        "healthchecks/<int:check_id>/trigger/",
        views.healthcheck_trigger,
        name="healthcheck_trigger",
    ),
    path(
        "healthchecks/<int:check_id>/toggle-disabled/",
        views.healthcheck_toggle_disabled,
        name="healthcheck_toggle_disabled",
    ),
    # Theme setting endpoint
    path("set-theme/", views.set_theme, name="set_theme"),
]



"""
Deployment tools for nyxmon.

Provides command-line utilities for deploying nyxmon to staging or production.
"""

import contextlib
import os
import subprocess
from pathlib import Path


@contextlib.contextmanager
def working_directory(path):
    """Changes working directory and returns to previous on exit."""
    prev_cwd = Path.cwd().absolute()
    try:
        os.chdir(path)
        yield
    finally:
        os.chdir(prev_cwd)


def _deploy(environment: str) -> None:
    """
    Use ansible-playbook to deploy the site to the specified environment.
    
    Args:
        environment: Either "staging" or "production"
    """
    # Find the deploy directory relative to the package
    # In a deployed package, this would be relative to the site root
    deploy_root = Path.cwd() / "deploy"
    if not deploy_root.exists():
        # Try to find it from the package location
        import nyxmon
        package_root = Path(nyxmon.__file__).parent.parent.parent
        deploy_root = package_root / "deploy"
        
    if not deploy_root.exists():
        raise FileNotFoundError(
            f"Could not find deploy directory at {deploy_root}. "
            "Make sure you're running this command from the project root."
        )
    
    with working_directory(deploy_root):
        subprocess.call(["ansible-playbook", "deploy.yml", "--limit", environment])


def deploy_staging() -> None:
    """
    Deploy nyxmon to the staging environment.
    """
    _deploy("staging")


def deploy_production() -> None:
    """
    Deploy nyxmon to the production environment.
    """
    _deploy("production")


def production_db_to_local() -> None:
    """
    Use ansible to create and fetch a database backup from production.
    
    Make sure only the database is running using:
      postgres -D databases/postgres
    """
    import psutil

    for proc in psutil.process_iter(["pid", "name", "username"]):
        if proc.info["name"] is None or "python" not in proc.info["name"]:
            continue
        try:
            cmdline = " ".join(proc.cmdline())
            if "honcho" in cmdline:
                print("Please stop honcho first and start a single postgres db with postgres -D databases/postgres")
                return
        except psutil.AccessDenied:
            # ignore processes that we cannot observe
            pass

    deploy_root = Path.cwd() / "deploy"
    if not deploy_root.exists():
        # Try to find it from the package location
        import nyxmon
        package_root = Path(nyxmon.__file__).parent.parent.parent
        deploy_root = package_root / "deploy"
        
    if not deploy_root.exists():
        raise FileNotFoundError(
            f"Could not find deploy directory at {deploy_root}. "
            "Make sure you're running this command from the project root."
        )
        
    with working_directory(deploy_root):
        output = subprocess.check_output(
            ["ansible-playbook", "backup_database.yml", "--limit", "production"], text=True
        )
    
    lines_with_backup = [line for line in output.split("\n") if "sql.gz" in line]
    if not lines_with_backup:
        print("No backup file found in ansible output")
        return
        
    backup_file_name = lines_with_backup[0].split('"')[-2]
    
    # Find the backups directory
    backups_dir = Path.cwd() / "backups"
    if not backups_dir.exists():
        import nyxmon
        package_root = Path(nyxmon.__file__).parent.parent.parent
        backups_dir = package_root / "backups"
    
    backup_path = backups_dir / backup_file_name
    db_name = "nyxmon"  # Changed from homepage to nyxmon
    
    subprocess.call(["dropdb", db_name])
    subprocess.call(["createdb", db_name])
    subprocess.call(["createuser", db_name])
    
    command = f"gunzip -c {backup_path} | psql {db_name}"
    print(command)
    subprocess.call(command, shell=True)
    print(f"Restored backup from {backup_path}")


"""
CLI entrypoints for check management operations.
"""

import argparse
import sys
import anyio
import asyncio
import time
import aiosqlite
from pathlib import Path
from datetime import datetime

from nyxmon.adapters.repositories import SqliteStore
from nyxmon.bootstrap import bootstrap
from nyxmon.domain import Check, CheckStatus
from nyxmon.domain.commands import AddCheck


# --- Add Check Functions ---


async def add_check_async(args):
    """Async function to add a check to the database."""
    # Validate database path
    db_path = Path(args.db)
    if not db_path.exists():
        print(f"Error: Database file not found: {db_path}")
        sys.exit(1)

    try:
        # Initialize store and message bus
        store = SqliteStore(db_path=db_path)
        bus = bootstrap(store=store)

        # Get next check ID if not provided
        if args.check_id is None:
            # Get all checks and find the highest ID - use async method
            existing_checks = await bus.uow.store.checks.list_async()
            existing_checks = False
            if existing_checks:
                max_id = max(check.check_id for check in existing_checks)
                check_id = max_id + 1
            else:
                check_id = 1
        else:
            check_id = args.check_id

        # Create the check
        check = Check(
            check_id=check_id,
            service_id=args.service_id,
            check_type="http",
            status=CheckStatus.IDLE,
            url=args.url,
            check_interval=args.interval,
            data={},
        )

        # Add the check using the message bus
        cmd = AddCheck(check=check)
        bus.handle(cmd)

        print(f"✓ Successfully added check ID {check_id}")
        print(f"  Service ID: {args.service_id}")
        print(f"  Type: {args.check_type}")
        print(f"  URL: {args.url}")
        print(f"  Interval: {args.interval} seconds")

    except Exception as e:
        print(f"Error adding check: {e}")
        raise e


def add_check_to_db():
    """CLI script to add a health check to the database."""
    parser = argparse.ArgumentParser(
        description="Add a health check to NyxMon database"
    )
    parser.add_argument("--db", required=True, help="Path to SQLite database file")
    parser.add_argument(
        "--service-id", type=int, required=True, help="Service ID for the check"
    )
    parser.add_argument(
        "--check-type",
        default="http",
        choices=["http", "tcp", "ping", "dns", "custom"],
        help="Type of health check (default: http)",
    )
    parser.add_argument("--url", required=True, help="URL or endpoint to check")
    parser.add_argument(
        "--interval",
        type=int,
        default=300,
        help="Check interval in seconds (default: 300)",
    )
    parser.add_argument(
        "--check-id",
        type=int,
        help="Specific check ID (will auto-increment if not provided)",
    )

    args = parser.parse_args()

    # Run the async function
    anyio.run(add_check_async, args)


# --- Show Checks Functions ---


def format_time(timestamp):
    """Format Unix timestamp to human-readable string."""
    if timestamp == 0:
        return "Never"
    return datetime.fromtimestamp(timestamp).strftime("%Y-%m-%d %H:%M:%S")


def format_seconds_ago(timestamp):
    """Format Unix timestamp as 'X seconds/minutes/hours ago'."""
    if timestamp == 0:
        return "Never"

    current_time = time.time()
    diff = current_time - timestamp

    if diff < 60:
        return f"{int(diff)} seconds ago"
    elif diff < 3600:
        minutes = int(diff / 60)
        return f"{minutes} minute{'s' if minutes > 1 else ''} ago"
    elif diff < 86400:
        hours = int(diff / 3600)
        return f"{hours} hour{'s' if hours > 1 else ''} ago"
    else:
        days = int(diff / 86400)
        return f"{days} day{'s' if days > 1 else ''} ago"


async def show_due_checks(db_path: Path):
    """Show all due checks from the database."""
    try:
        async with aiosqlite.connect(db_path) as db:
            db.row_factory = aiosqlite.Row

            # Get all checks with their service names
            query = """
            SELECT 
                hc.id as check_id,
                hc.service_id,
                hc.check_type,
                hc.url,
                hc.check_interval,
                hc.next_check_time,
                hc.processing_started_at,
                hc.status,
                s.name as service_name
            FROM health_check hc
            LEFT JOIN service s ON hc.service_id = s.id
            ORDER BY hc.next_check_time ASC
            """

            cursor = await db.execute(query)
            rows = await cursor.fetchall()
            rows_list = list(rows)  # Convert to list for type safety

            if not rows_list:
                print("No checks found in the database.")
                return

            current_time = int(time.time())
            due_checks = []
            upcoming_checks = []
            processing_checks = []

            # Categorize checks
            for row in rows_list:
                if row["status"] == CheckStatus.PROCESSING:
                    processing_checks.append(row)
                elif row["next_check_time"] <= current_time:
                    due_checks.append(row)
                else:
                    upcoming_checks.append(row)

            # Print results
            print("=" * 80)
            print(
                f"NyxMon Check Status Report - {datetime.now().strftime('%Y-%m-%d %H:%M:%S')}"
            )
            print("=" * 80)

            # Due checks
            if due_checks:
                print(f"\n📍 DUE CHECKS ({len(due_checks)}):")
                print("-" * 80)
                for check in due_checks:
                    print(f"  Check ID: {check['check_id']}")
                    print(
                        f"  Service:  {check['service_name']} (ID: {check['service_id']})"
                    )
                    print(f"  Type:     {check['check_type']}")
                    print(f"  URL:      {check['url']}")
                    print(f"  Due:      {format_seconds_ago(check['next_check_time'])}")
                    print(f"  Interval: {check['check_interval']} seconds")
                    print(f"  Status:   {check['status']}")
                    print("-" * 80)
            else:
                print("\n✅ No checks are currently due.")

            # Processing checks
            if processing_checks:
                print(f"\n⚡ PROCESSING CHECKS ({len(processing_checks)}):")
                print("-" * 80)
                for check in processing_checks:
                    print(f"  Check ID: {check['check_id']}")
                    print(
                        f"  Service:  {check['service_name']} (ID: {check['service_id']})"
                    )
                    print(f"  Type:     {check['check_type']}")
                    print(f"  URL:      {check['url']}")
                    print(
                        f"  Started:  {format_seconds_ago(check['processing_started_at'])}"
                    )
                    print(f"  Status:   {check['status']}")
                    print("-" * 80)

            # Upcoming checks
            if upcoming_checks:
                print("\n⏰ UPCOMING CHECKS (next 5):")
                print("-" * 80)
                for check in upcoming_checks[:5]:
                    time_until = check["next_check_time"] - current_time
                    if time_until < 60:
                        time_str = f"in {int(time_until)} seconds"
                    elif time_until < 3600:
                        time_str = f"in {int(time_until / 60)} minutes"
                    else:
                        time_str = f"in {int(time_until / 3600)} hours"

                    print(f"  Check ID: {check['check_id']}")
                    print(
                        f"  Service:  {check['service_name']} (ID: {check['service_id']})"
                    )
                    print(f"  Type:     {check['check_type']}")
                    print(f"  Next run: {time_str}")
                    print(f"  Status:   {check['status']}")
                    print("-" * 80)

            # Summary
            print("\nSUMMARY:")
            print(f"  Total checks: {len(rows_list)}")
            print(f"  Due now:      {len(due_checks)}")
            print(f"  Processing:   {len(processing_checks)}")
            print(f"  Upcoming:     {len(upcoming_checks)}")
            print("=" * 80)

    except Exception as e:
        print(f"Error reading database: {e}")
        sys.exit(1)


def show_checks():
    """CLI entrypoint for showing checks from the database."""
    parser = argparse.ArgumentParser(
        description="Show all due checks from NyxMon database"
    )
    parser.add_argument("--db", required=True, help="Path to SQLite database file")
    parser.add_argument(
        "--verbose", "-v", action="store_true", help="Show more detailed information"
    )

    args = parser.parse_args()

    # Validate database path
    db_path = Path(args.db)
    if not db_path.exists():
        print(f"Error: Database file not found: {db_path}")
        sys.exit(1)

    # Run the async function
    asyncio.run(show_due_checks(db_path))



from typing import Protocol, TypeAlias

from ...domain import Result, Check, Service


class ResultRepository(Protocol):
    """A repository interface for storing and retrieving results."""

    seen: set[Result]

    def add(self, result: Result) -> None:
        """Add a result to the repository."""
        ...

    def get(self, result_id: int) -> Result:
        """Get a result from the repository by ID."""
        ...

    def list(self) -> list[Result]:
        """Get a list of all results."""
        ...


class CheckRepository(Protocol):
    """A repository interface for storing and retrieving checks."""

    seen: set

    def add(self, check) -> None:
        """Add a check to the repository."""
        ...

    def get(self, check_id: int):
        """Get a check from the repository by ID."""
        ...

    def list(self) -> list[Check]:
        """Get a list of all checks."""
        ...


class ServiceRepository(Protocol):
    """A repository interface for storing and retrieving services."""

    seen: set

    def add(self, service) -> None:
        """Add a service to the repository."""
        ...

    def get(self, service_id: int):
        """Get a service from the repository by ID."""
        ...

    def list(self) -> list[Service]:
        """Get a list of all services."""
        ...


Repository: TypeAlias = ResultRepository | CheckRepository | ServiceRepository


class RepositoryStore(Protocol):
    """A protocol for a collection of repositories."""

    results: ResultRepository
    checks: CheckRepository
    services: ServiceRepository

    def list(self) -> list[Repository]:
        """Get a list of all repositories."""
        ...



# Generated by Django 5.2 on 2025-04-13 20:24

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="HealthCheck",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "check_type",
                    models.CharField(
                        choices=[
                            ("http", "HTTP"),
                            ("tcp", "TCP"),
                            ("ping", "Ping"),
                            ("dns", "DNS"),
                            ("custom", "Custom"),
                        ],
                        default="http",
                        max_length=20,
                        verbose_name="Check Type",
                    ),
                ),
                ("url", models.URLField(max_length=512, verbose_name="URL")),
                (
                    "check_interval",
                    models.IntegerField(
                        choices=[
                            (30, "30 seconds"),
                            (60, "1 minute"),
                            (300, "5 minutes"),
                            (900, "15 minutes"),
                            (1800, "30 minutes"),
                            (3600, "1 hour"),
                            (86400, "1 day"),
                        ],
                        default=300,
                        help_text="How often this health check should be performed",
                        verbose_name="Check Interval",
                    ),
                ),
            ],
            options={
                "db_table": "health_check",
            },
        ),
        migrations.CreateModel(
            name="Service",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, verbose_name="Service Name")),
            ],
            options={
                "db_table": "service",
            },
        ),
        migrations.CreateModel(
            name="Result",
            fields=[
                (
                    "id",
                    models.BigAutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "status",
                    models.CharField(
                        choices=[("ok", "OK"), ("error", "Error")],
                        max_length=10,
                        verbose_name="Status",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(auto_now_add=True, verbose_name="Created At"),
                ),
                (
                    "data",
                    models.JSONField(blank=True, default=dict, verbose_name="Metadata"),
                ),
                (
                    "health_check",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="results",
                        to="nyxboard.healthcheck",
                    ),
                ),
            ],
            options={
                "db_table": "check_result",
            },
        ),
        migrations.AddField(
            model_name="healthcheck",
            name="service",
            field=models.ForeignKey(
                on_delete=django.db.models.deletion.CASCADE, to="nyxboard.service"
            ),
        ),
    ]



from django.shortcuts import render, redirect, get_object_or_404
from django.http import JsonResponse
from django.views.decorators.http import require_POST
import json
from time import time

from .models import Service, HealthCheck, StatusChoices
from .forms import ServiceForm, HealthCheckForm
from nyxmon.domain import CheckStatus


def dashboard(request):
    """
    Function-based view to display the dashboard of services and their health checks.
    """
    # Get all services with their health checks
    services = Service.objects.prefetch_related("healthcheck_set")

    # Fetch recent results for all health checks separately
    health_checks = HealthCheck.objects.filter(service__in=services)

    # Dictionary to map check IDs to their last result
    check_results = {}

    # For each health check, fetch its recent results separately and determine mode
    current_time = time()

    for check in health_checks:
        # Get recent results
        recent_results = check.results.order_by("-created_at")[:5]
        check.recent_results = list(
            recent_results
        )  # Force evaluation and convert to list

        # Set check mode - determines if progress ring is shown or if it's due for a check
        if check.next_check_time <= current_time:
            check.check_mode = "due"
        else:
            check.check_mode = "normal"

        # Get last result if any
        if check.recent_results:
            check.last_result = check.recent_results[0]
            # Add to our mapping dictionary
            check_results[check.id] = {
                "formatted_time": check.last_result.created_at.strftime(
                    "%b %-d, %H:%M"
                ),
                "timestamp": int(check.last_result.created_at.timestamp()),
                "status": check.last_result.status,
            }
        else:
            check.last_result = None

    # Set the default theme if not in session
    if "theme" not in request.session:
        request.session["theme"] = "light"

    # Create a JSON object with only the essential check result data
    check_results_json = {}
    for check_id, result_data in check_results.items():
        check_results_json[str(check_id)] = {
            "formattedTime": result_data["formatted_time"]
        }

    context = {
        "services": services,
        "status_choices": StatusChoices,
        "theme": request.session.get("theme", "light"),
        "check_results_json": json.dumps(check_results_json),
        "check_results": check_results_json,
    }

    return render(request, "nyxboard/dashboard.html", context)


# Service CRUD views
def service_list(request):
    """
    Display a list of all services.
    """
    services = Service.objects.all()
    return render(request, "nyxboard/service_list.html", {"services": services})


def service_detail(request, service_id):
    """
    Display details of a specific service.
    """
    service = get_object_or_404(Service, id=service_id)
    health_checks = service.healthcheck_set.all()
    return render(
        request,
        "nyxboard/service_detail.html",
        {"service": service, "health_checks": health_checks},
    )


def service_create(request):
    """
    Create a new service.
    """
    if request.method == "POST":
        form = ServiceForm(request.POST)
        if form.is_valid():
            service = form.save()
            return redirect("nyxboard:service_detail", service_id=service.id)
    else:
        form = ServiceForm()

    return render(
        request, "nyxboard/service_form.html", {"form": form, "action": "Create"}
    )


def service_update(request, service_id):
    """
    Update an existing service.
    """
    service = get_object_or_404(Service, id=service_id)

    if request.method == "POST":
        form = ServiceForm(request.POST, instance=service)
        if form.is_valid():
            form.save()
            return redirect("nyxboard:service_detail", service_id=service.id)
    else:
        form = ServiceForm(instance=service)

    return render(
        request,
        "nyxboard/service_form.html",
        {"form": form, "service": service, "action": "Update"},
    )


def service_delete(request, service_id):
    """
    Delete a service.
    """
    service = get_object_or_404(Service, id=service_id)

    if request.method == "POST":
        service.delete()
        return redirect("nyxboard:service_list")

    return render(request, "nyxboard/service_confirm_delete.html", {"service": service})


# HealthCheck CRUD views
def healthcheck_list(request):
    """
    Display a list of all health checks.
    """
    health_checks = HealthCheck.objects.all()
    return render(
        request, "nyxboard/healthcheck_list.html", {"health_checks": health_checks}
    )


def healthcheck_detail(request, check_id):
    """
    Display details of a specific health check.
    """
    health_check = get_object_or_404(HealthCheck, id=check_id)
    results = health_check.results.order_by("-created_at")[:10]
    return render(
        request,
        "nyxboard/healthcheck_detail.html",
        {"health_check": health_check, "results": results},
    )


def healthcheck_create(request, service_id=None):
    """
    Create a new health check, optionally linked to a specific service.
    """
    initial = {}
    service = None

    if service_id:
        service = get_object_or_404(Service, id=service_id)
        initial["service"] = service

    if request.method == "POST":
        form = HealthCheckForm(request.POST)
        if form.is_valid():
            health_check = form.save()
            return redirect("nyxboard:healthcheck_detail", check_id=health_check.id)
    else:
        form = HealthCheckForm(initial=initial)

    return render(
        request,
        "nyxboard/healthcheck_form.html",
        {"form": form, "service": service, "action": "Create"},
    )


def healthcheck_update(request, check_id):
    """
    Update an existing health check.
    """
    health_check = get_object_or_404(HealthCheck, id=check_id)

    if request.method == "POST":
        # Check if this is a quick-toggle of the disabled flag from the card
        if (
            "disabled" in request.POST and len(request.POST) == 2
        ):  # Just disabled and csrf token
            new_disabled_value = request.POST.get("disabled") == "1"
            health_check.disabled = new_disabled_value
            health_check.save()

            # Redirect back to referring page, or dashboard if no referrer
            if request.META.get("HTTP_REFERER"):
                return redirect(request.META.get("HTTP_REFERER"))
            return redirect("nyxboard:dashboard")

        # Normal form submission
        form = HealthCheckForm(request.POST, instance=health_check)
        if form.is_valid():
            # Check if check_interval has changed
            if "check_interval" in form.changed_data:
                # Reset next_check_time to now to make the check due immediately
                health_check = form.save(commit=False)
                health_check.next_check_time = int(time())
                health_check.save()
            else:
                form.save()
            return redirect("nyxboard:healthcheck_detail", check_id=health_check.id)
    else:
        form = HealthCheckForm(instance=health_check)

    return render(
        request,
        "nyxboard/healthcheck_form.html",
        {"form": form, "health_check": health_check, "action": "Update"},
    )


def healthcheck_delete(request, check_id):
    """
    Delete a health check.
    """
    health_check = get_object_or_404(HealthCheck, id=check_id)

    if request.method == "POST":
        service_id = health_check.service.id
        health_check.delete()
        return redirect("nyxboard:service_detail", service_id=service_id)

    return render(
        request,
        "nyxboard/healthcheck_confirm_delete.html",
        {"health_check": health_check},
    )


# HTMX-enabled views for health check updates
def healthcheck_update_status(request, check_id):
    """
    Update the status of a health check for HTMX updates.
    This view is called periodically to check if a health check's status has changed.
    """
    health_check = get_object_or_404(HealthCheck, id=check_id)
    recent_results = health_check.results.order_by("-created_at")[:5]

    # Attach needed data to the health check for the template
    health_check.recent_results = recent_results

    # Determine if it's still due or back to normal
    current_time = time()

    # Set last_result regardless of status
    last_result = recent_results[0] if recent_results else None

    if health_check.status == CheckStatus.PROCESSING:
        # If it's being processed, keep in due mode
        check_mode = "due"
    elif health_check.next_check_time <= current_time:
        # If it's past next check time, it's still due
        check_mode = "due"
    else:
        # If next check time is in the future, it's back to normal
        check_mode = "normal"

    # Determine which template to use based on what partial was requested
    template_name = "nyxboard/partials/healthcheck-card.html"
    if "healthcheck.html" in request.headers.get("HX-Request-URL", ""):
        template_name = "nyxboard/partials/healthcheck.html"

    context = {
        "check": health_check,
        "check_mode": check_mode,
        "last_result": last_result,
        "theme": request.session.get("theme", "light"),
    }

    return render(request, template_name, context)


def healthcheck_trigger(request, check_id):
    """
    Manually trigger a health check to be run now.
    This marks the check as due immediately.
    """
    if request.method != "POST":
        return redirect("nyxboard:dashboard")

    health_check = get_object_or_404(HealthCheck, id=check_id)

    # Get data needed for the template first
    recent_results = health_check.results.order_by("-created_at")[:5]
    last_result = recent_results[0] if recent_results else None
    health_check.recent_results = recent_results

    # Set the next check time to now, so it will be picked up by the agent
    health_check.next_check_time = int(time())
    health_check.save()

    # Determine which template to use based on what partial was requested
    template_name = "nyxboard/partials/healthcheck-card.html"
    if "healthcheck.html" in request.headers.get("HX-Request-URL", ""):
        template_name = "nyxboard/partials/healthcheck.html"

    context = {
        "check": health_check,
        "check_mode": "due",
        "last_result": last_result,  # Use the stored last_result variable
        "theme": request.session.get("theme", "light"),
    }

    return render(request, template_name, context)


def healthcheck_toggle_disabled(request, check_id):
    """
    Toggle the disabled status of a health check.
    Uses HTMX to update the check card.
    """
    if request.method != "POST":
        return redirect("nyxboard:dashboard")

    health_check = get_object_or_404(HealthCheck, id=check_id)

    # Get data needed for the template first
    recent_results = health_check.results.order_by("-created_at")[:5]
    last_result = recent_results[0] if recent_results else None
    health_check.recent_results = recent_results

    # Toggle the disabled status
    health_check.disabled = not health_check.disabled

    # If we're disabling, reset the next check time to now
    # This prevents the progress ring from showing progress for disabled checks
    if health_check.disabled:
        health_check.next_check_time = int(time())

    health_check.save()

    # Determine check mode based on current status
    current_time = time()
    if health_check.disabled:
        check_mode = "normal"  # Don't show due status for disabled checks
    elif health_check.next_check_time <= current_time:
        check_mode = "due"
    else:
        check_mode = "normal"

    # Determine which template to use based on what partial was requested
    template_name = "nyxboard/partials/healthcheck-card.html"
    if "healthcheck.html" in request.headers.get("HX-Request-URL", ""):
        template_name = "nyxboard/partials/healthcheck.html"

    context = {
        "check": health_check,
        "check_mode": check_mode,
        "last_result": last_result,
        "theme": request.session.get("theme", "light"),
    }

    return render(request, template_name, context)


@require_POST
def set_theme(request):
    """
    Set the theme preference in the session.
    """
    try:
        data = json.loads(request.body)
        theme = data.get("theme", "light")
        request.session["theme"] = theme
        return JsonResponse({"status": "success", "theme": theme})
    except json.JSONDecodeError:
        return JsonResponse({"status": "error", "message": "Invalid JSON"}, status=400)



# Generated by Django 5.2.1 on 2025-05-17 11:16

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nyxboard", "0004_healthcheck_status"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthcheck",
            name="name",
            field=models.CharField(
                default="",
                help_text="A descriptive name for this health check",
                max_length=255,
                verbose_name="Check Name",
            ),
            preserve_default=False,
        ),
    ]



import anyio
from anyio import to_thread

from anyio.from_thread import BlockingPortalProvider
from typing import Iterable, Callable

import httpx

from .interface import CheckRunner
from ...domain import Check, Result, ResultStatus


class AsyncCheckRunner(CheckRunner):
    def __init__(self, portal_provider: BlockingPortalProvider) -> None:
        self.portal_provider = portal_provider

    def run_all(self, checks: Iterable[Check], result_received: Callable) -> None:
        """Run all checks."""

        async def run_checks(result_received_callback: Callable) -> None:
            async for result in self._async_run_all(checks):
                # Process the result in a worker thread
                await to_thread.run_sync(result_received_callback, result)

        # Run the async function in the portal
        with self.portal_provider as portal:
            portal.call(run_checks, result_received)

    async def _async_run_all(self, checks: Iterable[Check]):
        send_channel: anyio.abc.ObjectSendStream[Result]
        receive_channel: anyio.abc.ObjectReceiveStream[Result]
        send_channel, receive_channel = anyio.create_memory_object_stream(
            max_buffer_size=100
        )

        async with send_channel, receive_channel:
            async with httpx.AsyncClient(follow_redirects=True, timeout=10) as client:
                async with anyio.create_task_group() as tg:
                    for check in checks:
                        tg.start_soon(self._run_one, client, check, send_channel)

                # task group finishes -> all _run_one are done
                await send_channel.aclose()

            async for result in receive_channel:
                yield result

    async def _run_one(
        self,
        client: httpx.AsyncClient,
        check: Check,
        send_channel: anyio.abc.ObjectSendStream,
    ) -> None:
        try:
            r = await client.get(check.url)
            r.raise_for_status()
            result = Result(check_id=check.check_id, status=ResultStatus.OK, data={})
        except httpx.HTTPStatusError as e:
            result = Result(
                check_id=check.check_id,
                status=ResultStatus.ERROR,
                data={"error_msg": str(e), "status_code": e.response.status_code},
            )
        except httpx.ConnectError as e:
            result = Result(
                check_id=check.check_id,
                status=ResultStatus.ERROR,
                data={"error_type": "connection_error", "error_msg": str(e)},
            )
        except httpx.RequestError as e:
            result = Result(
                check_id=check.check_id,
                status=ResultStatus.ERROR,
                data={"error_type": "request_error", "error_msg": str(e)},
            )
        except Exception as e:
            # Catch-all for any other exceptions
            result = Result(
                check_id=check.check_id,
                status=ResultStatus.ERROR,
                data={"error_msg": str(e)},
            )

        await send_channel.send(result)



from time import time

from django.db import models

from nyxmon.domain import ResultStatus, StatusChoices, CheckStatus, CheckType


class Service(models.Model):
    name: models.CharField = models.CharField("Service Name", max_length=255)

    class Meta:
        db_table = "service"

    def __str__(self):
        return self.name

    def get_status(self):
        """
        Calculate the overall status of the service based on its health checks.
        """
        health_checks = self.healthcheck_set.all()

        if not health_checks.exists():
            return StatusChoices.UNKNOWN

        statuses = [check.get_status() for check in health_checks]

        if StatusChoices.FAILED in statuses:
            return StatusChoices.FAILED

        if StatusChoices.WARNING in statuses or StatusChoices.RECOVERING in statuses:
            return StatusChoices.WARNING

        if all(status == StatusChoices.PASSED for status in statuses):
            return StatusChoices.PASSED

        if all(status == StatusChoices.UNKNOWN for status in statuses):
            return StatusChoices.UNKNOWN

        return StatusChoices.WARNING


class HealthCheck(models.Model):
    CHECK_TYPE_CHOICES = [
        (CheckType.HTTP, "HTTP"),
        (CheckType.JSON_HTTP, "JSON-HTTP"),
        (CheckType.TCP, "TCP"),
        (CheckType.PING, "Ping"),
        (CheckType.DNS, "DNS"),
        (CheckType.CUSTOM, "Custom"),
    ]

    INTERVAL_CHOICES = [
        (30, "30 seconds"),
        (60, "1 minute"),
        (300, "5 minutes"),
        (900, "15 minutes"),
        (1800, "30 minutes"),
        (3600, "1 hour"),
        (86400, "1 day"),
    ]

    STATUS_CHOICES = [
        (CheckStatus.IDLE, "Idle"),
        (CheckStatus.PROCESSING, "Processing"),
    ]

    name: models.CharField = models.CharField(
        "Check Name",
        max_length=255,
        help_text="A descriptive name for this health check",
    )
    service: models.ForeignKey = models.ForeignKey(Service, on_delete=models.CASCADE)
    check_type: models.CharField = models.CharField(
        "Check Type", max_length=20, choices=CHECK_TYPE_CHOICES, default="http"
    )
    url: models.URLField = models.URLField("URL", max_length=512)
    check_interval: models.IntegerField = models.IntegerField(
        "Check Interval",
        choices=INTERVAL_CHOICES,
        default=300,
        help_text="How often this health check should be performed",
    )
    status: models.CharField = models.CharField(
        "Status",
        max_length=10,
        choices=STATUS_CHOICES,
        default=CheckStatus.IDLE,
    )
    next_check_time: models.PositiveIntegerField = models.PositiveIntegerField(
        "Next Check Time", default=0, help_text="Unix timestamp of the next check"
    )
    processing_started_at: models.PositiveIntegerField = models.PositiveIntegerField(
        "Processing Started At",
        default=0,
        help_text="Unix timestamp when the check started processing",
    )
    disabled: models.BooleanField = models.BooleanField(
        "Disabled",
        default=False,
        help_text="If checked, this health check will not be executed",
    )

    class Meta:
        db_table = "health_check"

    def __str__(self):
        return f"{self.name} ({self.get_check_type_display()} Check {self.id})"

    def get_status(self):
        """
        Calculate the health check status based on recent results.
        """
        # Use recent_results if it's available (set by the dashboard view)
        # Otherwise, query the database
        if hasattr(self, "recent_results"):
            recent_results = self.recent_results
        else:
            # Get the 5 most recent results
            recent_results = self.results.order_by("-created_at")[:5]

        if not recent_results:
            return StatusChoices.UNKNOWN

        latest_result = recent_results[0] if recent_results else None

        if not latest_result:
            return StatusChoices.UNKNOWN

        # Check for Failed status (the latest result is error)
        if latest_result.status == ResultStatus.ERROR:
            return StatusChoices.FAILED

        # Check for Passed status (all recent results are OK)
        if all(result.status == ResultStatus.OK for result in recent_results):
            return StatusChoices.PASSED

        # Check for Recovering status (latest is OK but there were recent errors)
        if latest_result.status == ResultStatus.OK and any(
            result.status == ResultStatus.ERROR for result in recent_results
        ):
            return StatusChoices.RECOVERING

        # Otherwise, it's a Warning status
        return StatusChoices.WARNING

    @property
    def percentage_until_next_check(self):
        """
        Calculate the percentage of time until the next check.
        """
        if self.check_interval == 0:
            return 0
        current_time = time()
        return 100 - round(
            (self.next_check_time - current_time) / self.check_interval * 100
        )


class Result(models.Model):
    health_check: models.ForeignKey = models.ForeignKey(
        HealthCheck, on_delete=models.CASCADE, related_name="results"
    )
    status: models.CharField = models.CharField(
        "Status",
        max_length=10,
        choices=[(ResultStatus.OK, "OK"), (ResultStatus.ERROR, "Error")],
    )
    created_at: models.DateTimeField = models.DateTimeField(
        "Created At", auto_now_add=True
    )
    data = models.JSONField("Metadata", blank=True, default=dict)

    class Meta:
        db_table = "check_result"

    def __str__(self):
        return f"Result {self.id} for {self.health_check} ({self.status})"



from django import forms

from .models import Service, HealthCheck


class ServiceForm(forms.ModelForm):
    """Form for creating and updating Service objects."""

    class Meta:
        model = Service
        fields = ["name"]
        widgets = {"name": forms.TextInput(attrs={"class": "form-control"})}


class HealthCheckForm(forms.ModelForm):
    """Form for creating and updating HealthCheck objects."""

    class Meta:
        model = HealthCheck
        fields = ["name", "service", "check_type", "url", "check_interval", "disabled"]
        widgets = {
            "name": forms.TextInput(
                attrs={
                    "class": "form-control",
                    "placeholder": "e.g. Homepage Availability",
                }
            ),
            "service": forms.Select(attrs={"class": "form-control"}),
            "check_type": forms.Select(attrs={"class": "form-control"}),
            "url": forms.URLInput(
                attrs={
                    "class": "form-control",
                    "placeholder": "https://example.com/api/health",
                }
            ),
            "check_interval": forms.Select(attrs={"class": "form-control"}),
            "disabled": forms.CheckboxInput(attrs={"class": "form-check-input"}),
        }
        help_texts = {
            "name": "A descriptive name for this health check.",
            "check_type": "Select the type of health check to perform.",
            "url": "Enter the URL to check for HTTP health checks.",
            "check_interval": "Select how frequently this health check should run.",
            "disabled": "Check this box to temporarily disable this health check without deleting it.",
        }



# Generated by Django 5.2.1 on 2025-05-18 20:29

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nyxboard", "0005_add_name"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthcheck",
            name="disabled",
            field=models.BooleanField(
                default=False,
                help_text="Disable this health check without deleting it",
                verbose_name="Disabled",
            ),
        ),
    ]



from typing import Self, TYPE_CHECKING

from ..domain import Command

if TYPE_CHECKING:
    from .message_bus import Message

from ..adapters.repositories import RepositoryStore, InMemoryStore


class UnitOfWork:
    def __init__(self, store: RepositoryStore = InMemoryStore()) -> None:
        self.store = store
        self._in_transaction = False
        self._new_messages: list["Message"] = []

    def __enter__(self) -> Self:
        # Start transaction if supported by the repository
        if hasattr(self.store, "connection"):
            self.store.connection.execute("BEGIN TRANSACTION")
            self._in_transaction = True
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if exc_type:
            self.rollback()
        else:
            self.commit()

    def commit(self):
        if self._in_transaction and hasattr(self.store, "connection"):
            self.store.connection.commit()
            self._in_transaction = False

    def rollback(self):
        if self._in_transaction and hasattr(self.store, "connection"):
            self.store.connection.rollback()
            self._in_transaction = False

    def add_command(self, command: Command) -> None:
        """Add a command to the unit of work."""
        self._new_messages.append(command)

    def collect_new_events(self):
        """Collect all new events from the store."""
        # collect events for seen checks
        while self.store.checks.seen:
            check = self.store.checks.seen.pop()
            while check.events:
                yield check.events.pop()
        # collect new messages
        while self._new_messages:
            yield self._new_messages.pop()
        # for repository in self.store.list():
        #     for aggregate in repository.list():
        #         while aggregate.events:
        #             yield aggregate.events.pop(0)



"""
Management command to create development data for testing.
"""

from django.core.management.base import BaseCommand, CommandError
from django.db import transaction

from nyxboard.devdata import create_sample_data


class Command(BaseCommand):
    help = "Creates sample development data (services and checks) for testing"

    def add_arguments(self, parser):
        parser.add_argument(
            "--force",
            action="store_true",
            help="Force creation even if data already exists",
        )

    @transaction.atomic
    def handle(self, *args, **options):
        from nyxboard.models import Service, HealthCheck

        # Check if data already exists
        existing_services = Service.objects.count()
        existing_checks = HealthCheck.objects.count()

        if existing_services > 0 or existing_checks > 0:
            if not options["force"]:
                self.stdout.write(
                    self.style.WARNING(
                        f"Data already exists: {existing_services} services, "
                        f"{existing_checks} checks. Use --force to override."
                    )
                )
                return
            self.stdout.write(
                self.style.WARNING(
                    f"Proceeding with data creation despite existing data: "
                    f"{existing_services} services, {existing_checks} checks."
                )
            )

        # Create the sample data
        try:
            created = create_sample_data()

            # Report what was created
            self.stdout.write(
                self.style.SUCCESS(
                    f"Successfully created {len(created['services'])} services "
                    f"and {len(created['checks'])} checks:"
                )
            )

            for service in created["services"]:
                self.stdout.write(f"  - Service: {service.name} (ID: {service.id})")

            for check in created["checks"]:
                self.stdout.write(
                    f"  - Check: {check.name} (ID: {check.id}) "
                    f"for {check.service.name} - URL: {check.url}"
                )

            self.stdout.write(
                self.style.SUCCESS(
                    "You can now run the monitoring agent to start checking these endpoints"
                )
            )

        except Exception as e:
            raise CommandError(f"Error creating development data: {e}")



import anyio
import argparse
import logging
import sys
import uvloop

from pathlib import Path

from nyxmon.adapters.collector import running_collector, AsyncCheckCollector
from ..bootstrap import bootstrap
from ..adapters.repositories import SqliteStore
from ..adapters.notification import AsyncTelegramNotifier, LoggingNotifier

logger = logging.getLogger(__name__)


def setup_logging(log_level: str):
    logging.basicConfig(
        level=getattr(logging, log_level),
        format="%(asctime)s - %(name)s - %(levelname)s - %(message)s",
    )


def signal_handler(agent, signum, frame):
    logger.info(f"Received signal {signum}, shutting down...")
    agent.stop()


async def run_agent_with_collector(store, interval, enable_telegram=False):
    """Async function to run the agent with collector"""
    if enable_telegram:
        notifier = AsyncTelegramNotifier()
        logger.info("Telegram notifications enabled")
    else:
        notifier = LoggingNotifier()

    bus = bootstrap(
        store=store, collector=AsyncCheckCollector(interval=interval), notifier=notifier
    )

    async with running_collector(bus):
        logger.info(f"Agent started with {interval}s check interval")

        # Wait forever until cancelled (e.g., by Ctrl+C)
        try:
            await anyio.sleep_forever()
        except BaseException:
            logger.info("Agent shutting down...")
            raise


def start_agent():
    """CLI entrypoint for starting the monitoring agent."""
    parser = argparse.ArgumentParser(description="Run the NyxMon monitoring agent")
    parser.add_argument("--db", required=True, help="Path to SQLite database file")
    parser.add_argument(
        "--interval", type=int, default=5, help="Check interval in seconds (default: 5)"
    )
    parser.add_argument(
        "--log-level",
        default="INFO",
        choices=["DEBUG", "INFO", "WARNING", "ERROR", "CRITICAL"],
        help="Set the logging level (default: INFO)",
    )
    parser.add_argument(
        "--enable-telegram",
        action="store_true",
        help="Enable Telegram notifications (requires TELEGRAM_BOT_TOKEN and TELEGRAM_CHAT_ID env vars)",
    )

    args = parser.parse_args()

    # Configure logging
    setup_logging(args.log_level)

    # Validate database path
    db_path = Path(args.db)
    if not db_path.exists():
        logger.error(f"Database file not found: {db_path}")
        sys.exit(1)

    try:
        logger.info(f"Initializing agent with database: {args.db}")
        store = SqliteStore(db_path=db_path)

        async def main():
            await run_agent_with_collector(
                store, args.interval, enable_telegram=args.enable_telegram
            )

        # anyio automatically handles SIGINT and SIGTERM
        anyio.run(main, backend_options={"loop_factory": uvloop.new_event_loop})

    except KeyboardInterrupt:
        logger.info("Agent stopped by user")
    except Exception:
        logger.exception("Error running agent")
        sys.exit(1)


if __name__ == "__main__":
    start_agent()



# Generated by Django 5.2 on 2025-05-10 21:10

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ("nyxboard", "0003_healthcheck_processing_started_at"),
    ]

    operations = [
        migrations.AddField(
            model_name="healthcheck",
            name="status",
            field=models.CharField(
                choices=[("idle", "Idle"), ("processing", "Processing")],
                default="idle",
                max_length=10,
                verbose_name="Status",
            ),
        ),
    ]
